require 'inits'
require 'thor'
require 'rake'

# Commands:
#  inits help [COMMAND]
#   - Describe available commands or one specific...
#  inits on {brew_path=default is /usr/local/bin} {rbenv_path=default is anyenv}
#   - init process with logon
module Inits
  class CLI < Thor
    class_option :verbose, type: :boolean, aliases: '-v'
    class_option :brew_path, default: '/usr/local/bin/brew'
    class_option :rbenv_path, default: '~/.anyenv/envs/rbenv'

    desc 'help', 'this commands help document'
    def help
      puts <<-HELP
      Manual:
        Commands:
          bundle exec exe/inits on

        Options:
          -v: verbose
          -brew_path: brew's local path, default is `#{options[:brew_path]}`
          -rbenv_path: rbenv's local path, default is `#{options[:rbenv_path]}`
          -rbenv_installed: rbenv's local installed ruby versions, default is `true`
          -rbenv_listing: rbenv's all ruby versions, default is `false`
      HELP
    end

    method_option :rbenv_installed, type: :boolean, default: true
    method_option :rbenv_listing, type: :boolean, default: false
    desc 'on {brew_path=default is /usr/local/bin} {rbenv_path=default is anyenv}', 'init process with logon'
    def on
      p options if options[:verbose]
      brew
      rbenv
    end

    private

    def execute(label)
      return if options["#{label}_path".to_sym].empty?
      puts "[#{label}] update start..."
      yield if block_given?
      puts "[#{label}] update end...\n\n"
    end

    def brew
      execute('brew') { system "#{options[:brew_path]} update" }
    end

    def rbenv
      pullmaster = ->(*paths) { paths.map { |path| system "cd #{path} && git pull origin master" } }
      execute('rbenv') do
        pullmaster.call options[:rbenv_path], "#{options[:rbenv_path]}/plugins/ruby-build"
        system 'rbenv versions && ruby -v' if options[:rbenv_installed]
        system 'rbenv install --list' if options[:rbenv_listing]
      end
    end
  end
end
